package Getopt::EX::i18n::iso3361;
use 5.014;
use strict;
use warnings;
use Exporter 'import';

our @EXPORT_OK = qw(%iso3361);

##
## This data is generated by Country::Codes module.
##

our %iso3361 = (
    AM => "Armenia",
    AT => "Austria",
    AU => "Australia",
    BE => "Belgium",
    BG => "Bulgaria",
    BR => "Brazil",
    BY => "Belarus",
    CA => "Canada",
    CH => "Switzerland",
    CN => "China",
    CZ => "Czech Republic",
    DE => "Germany",
    DK => "Denmark",
    EE => "Estonia",
    ES => "Spain",
    ET => "Ethiopia",
    FI => "Finland",
    FR => "France",
    GB => "United Kingdom",
    GR => "Greece",
    HK => "Hong Kong",
    HR => "Croatia",
    HU => "Hungary",
    IE => "Ireland",
    IL => "Israel",
    IS => "Iceland",
    IT => "Italy",
    JP => "Japan",
    KR => "Korea, Republic of",
    KZ => "Kazakhstan",
    LT => "Lithuania",
    NL => "Netherlands",
    NO => "Norway",
    NZ => "New Zealand",
    PL => "Poland",
    PT => "Portugal",
    RO => "Romania",
    RU => "Russian Federation",
    SE => "Sweden",
    SI => "Slovenia",
    SK => "Slovakia",
    TR => "Turkey",
    TW => "Taiwan, Province of China",
    UA => "Ukraine",
    US => "United States",
    YU => "Yugoslavia",
    ZA => "South Africa",
    );

1;
